
require(afex)     # for mixed models and reshape2
require(stringr)  # for string operations
require(ascii)    # for nice tables
options(asciiType="org")
require(multcomp) # for post-hoc tests

# parameters:
inferences <- c("MP", "MT", "AC", "DA")

## load and prepare data
p1.files <- list.files("data", "rmc1_probabilistic_", full.names=TRUE)
p1.in <- do.call("rbind", lapply(p1.files, read.csv2, dec = ".", fileEncoding = "UTF-8"))

p1.l <- within(p1.in, {
  trial <- factor(trial)
  item <- factor(item)
  inference <- factor(inference, levels = inferences)
  X <- NULL
})

#levels(p1.l$what)
#levels(p1.l$probability)

p1 <- dcast(p1.l, code + item + inference ~ type + what, value.var = "resp")
p1 <- within(p1, {
  p.and.q <- ifelse(is.na(probability_p.and.q),  probability_PandQ, probability_p.and.q)
  conditional.p <- ifelse(is.na(probability2_conditional.probability), probability2_conditionalProbability,  probability2_conditional.probability)
  probability2_conditional.probability <- NULL
  probability2_conditionalProbability <- NULL
  probability_p.and.q <- NULL
  probability_PandQ <- NULL
})
colnames(p1)[4:9] <- c("p.inference", "p.not.p.or.q", "p.p.or.q", "p.premise", "p.alternative", "p.conditional")
# p.inference = probability of the conclusion following the inference, type (b) in paper
# p.p.or.q = type (c), last item

p2 <- p1[,1:3]
p2 <- cbind(p2, p1[,-(1:3)]/100)
head(p2)


#############
# conditional prob hypothesis
#############

nice.corr.table <- function(matrix, caption, max.p = 0.05) {
  #browser()
  rs <- Hmisc::rcorr(matrix)
  tmp.cor <- ifelse(rs[["r"]] < 0, paste("-", substr(formatC(rs[["r"]], digits=2, format="f"), 3, 5), sep = ""), substr(formatC(rs[["r"]], digits=2, format="f"), 2, 4))
  tmp.p <- rs[["P"]]
  #keep the following line if you want to have adjusted p.values marked only.
  tmp.p <- p.adjust(tmp.p, method = "holm")
  dim(tmp.p) <- dim(rs[["P"]])
  dimnames(tmp.p) <- dimnames(rs[["P"]])
  tmp.cor <- ifelse(tmp.p < max.p, paste("\\textbf{", tmp.cor, "}", sep = ""), tmp.cor)
  tmp.cor[lower.tri(tmp.cor)] <- NA
  tmp.cor <- as.data.frame(tmp.cor)
  tmp.cor <- tmp.cor[-nrow(tmp.cor),-1]
  suppressWarnings(print(ascii(tmp.cor, include.rownames = TRUE, include.colnames = TRUE, caption = caption)))
}


nice.corr.table(as.matrix(p2[, c("p.conditional", "conditional.p", "p.and.q", "p.not.p.or.q", "p.alternative")]), "Correlations (over all participants, each participants enters 4 data points)")  

# When writing the paper, Nelder_Mead was the default optimizer. Hence:
lmer.controls <- lmerControl(optCtrl = list(maxfun = 50000), optimizer="Nelder_Mead")
#prob.c1 <- mixed(p.conditional ~ conditional.p + p.and.q + p.not.p.or.q + p.alternative + (1|code) + (1|item), p2, control = lmer.controls, test.intercept=TRUE)
prob.c2 <- mixed(p.conditional ~ inference + conditional.p + p.and.q + p.not.p.or.q + p.alternative + (1|code) + (inference|item), p2, control = lmer.controls, test.intercept=TRUE)
#p1.s <- cbind(p2[,1:3], as.data.frame(scale(p2[,-(1:3)], scale = FALSE)))
#prob.c2b <- mixed(p.conditional ~ inference + conditional.p + p.and.q + p.not.p.or.q + p.alternative + (1|code) + (inference|item), p1.s, control = lmer.controls)
# prob.c3 <- mixed(p.conditional ~ conditional.p*p.and.q*p.not.p.or.q*p.alternative + (1|code) + (1|item), p1.s, control = lmer.controls)
# prob.c4 <- mixed(p.conditional ~ inference + conditional.p*p.and.q*p.not.p.or.q*p.alternative + (1|code) + (inference|item), p1.s, control = lmer.controls)  # no warnings

res.table <- summary(prob.c2)[,1:6]
res.table <- cbind(res.table, parameter = fixef(prob.c2[["full.model"]])[-(3:4)])
res.table[2,"parameter"] <- NA
res.table$p.value <- round_ps(res.table$p.value)
res.table$df <- paste(substr(res.table$ndf, 1, 1), formatC(res.table$ddf, digits = 2, format = "f"), sep = ", ")
res.table$F.scaling <- prettyNum(res.table$F.scaling, digits = 2)
res.table <- res.table[,c("Effect", "parameter", "F", "df", "F.scaling", "p.value")]
suppressWarnings(print(ascii(res.table, include.rownames = FALSE, caption = str_c("LMM on the probability of the conditional"))))


#############
# p-validity
#############

p2s <- p2
p2s$u.premises <- with(p2s, (1 - p.conditional) + (1 - p.premise))
p2s$u.premises <- ifelse(p2s$u.premises > 1, 1, p2s$u.premises)

p2s$u.conclusion <- 1 - p2s$p.inference
p2s$p.valid <- as.numeric(p2s$u.conclusion > p2s$u.premises)
p2s$p.valid.hit <- as.numeric(p2s$u.conclusion <= p2s$u.premises)
p2s$diff.0 <- p2s$p.valid.hit - p2s$u.premises

p2s$u.premises.5 <- p2s$u.premises + .05
p2s$u.premises.5 <- ifelse(p2s$u.premises.5 > 1, 1, p2s$u.premises.5)
p2s$p.valid.hit.5 <- as.numeric(p2s$u.conclusion <= p2s$u.premises.5)
p2s$diff.5 <- p2s$p.valid.hit.5 - p2s$u.premises.5

p2s$u.premises.10 <- p2s$u.premises + .10
p2s$u.premises.10 <- ifelse(p2s$u.premises.10 > 1, 1, p2s$u.premises.10)
p2s$p.valid.hit.10 <- as.numeric(p2s$u.conclusion <= p2s$u.premises.10)
p2s$diff.10 <- p2s$p.valid.hit.10 - p2s$u.premises.10

  
p2sv <- droplevels(subset(p2s, inference %in% c("MP", "MT")))

(lmm1.pvalid <- mixed(diff.0 ~ inference + (1|code) + (1|item), p2sv, control = lmer.controls, test.intercept=TRUE))

K2 <- matrix(c(1, 1, 1, -1), 2)
rownames(K2) <- c("MP", "MT")
summary(glht(lmm1.pvalid[[2]], linfct = K2, alternative = "greater"), test = adjusted("holm"))
summary(glht(lmm1.pvalid[[2]], linfct = K2, alternative = "greater"), test = adjusted("none"))

(lmm2.pvalid <- mixed(diff.5 ~ inference + (1|code) + (1|item), p2sv, control = lmer.controls))
summary(glht(lmm2.pvalid[[2]], linfct = K2, alternative = "greater"), test = adjusted("holm"))

(lmm3.pvalid <- mixed(diff.10 ~ inference + (1|code) + (inference|item), p2sv, control = lmer.controls))
summary(glht(lmm3.pvalid[[2]], linfct = K2, alternative = "greater"), test = adjusted("holm"))


#############
# Coherence
#############

head(p2)
p1.s <- p2
p1.s$inference.ordered <- ordered(p1.s$inference)

p1.s$lower.b <- NA
p1.s$upper.b <- NA

## calculate coherence bounds:
p1.s[p1.s$inference == "MP", "lower.b"] <- p1.s[p1.s$inference == "MP", "p.conditional"]*p1.s[p1.s$inference == "MP", "p.premise"]
p1.s[p1.s$inference == "MT", "lower.b"] <- apply(cbind((1 - p1.s[p1.s$inference == "MT", "p.conditional"]  - p1.s[p1.s$inference == "MT", "p.premise"]) / (1 - p1.s[p1.s$inference == "MT", "p.conditional"]), (p1.s[p1.s$inference == "MT", "p.conditional"] + p1.s[p1.s$inference == "MT", "p.premise"] - 1)/p1.s[p1.s$inference == "MT", "p.conditional"]), 1, max)
p1.s[p1.s$inference == "AC", "lower.b"] <- 0
p1.s[p1.s$inference == "DA", "lower.b"] <- 1 - p1.s[p1.s$inference == "DA", "p.premise"] - p1.s[p1.s$inference == "DA", "p.conditional"]*(1 - p1.s[p1.s$inference == "DA", "p.premise"])

p1.s[p1.s$inference == "MP", "upper.b"] <- 1 - p1.s[p1.s$inference == "MP", "p.premise"] + p1.s[p1.s$inference == "MP", "p.conditional"]*p1.s[p1.s$inference == "MP", "p.premise"]
p1.s[p1.s$inference == "MT", "upper.b"] <- 1
p1.s[p1.s$inference == "AC", "upper.b"] <- pmin(p1.s[p1.s$inference == "AC", "p.premise"]/p1.s[p1.s$inference == "AC", "p.conditional"], (1 - p1.s[p1.s$inference == "AC", "p.premise"]) /  (1 - p1.s[p1.s$inference == "AC", "p.conditional"]), na.rm = TRUE)
p1.s[p1.s$inference == "DA", "upper.b"] <- 1 - p1.s[p1.s$inference == "DA", "p.conditional"]*(1 - p1.s[p1.s$inference == "DA", "p.premise"])

p1.s$lower.5 <- p1.s$lower.b - 0.05
p1.s$upper.5 <- p1.s$upper.b + 0.05
p1.s$lower.10 <- p1.s$lower.b - 0.1
p1.s$upper.10 <- p1.s$upper.b + 0.1

p1.s$lower.5 <- ifelse(p1.s$lower.5 < 0, 0, p1.s$lower.5)
p1.s$lower.10 <- ifelse(p1.s$lower.10 < 0, 0, p1.s$lower.10)
p1.s$upper.5 <- ifelse(p1.s$lower.5 > 1, 1, p1.s$upper.5)
p1.s$upper.10 <- ifelse(p1.s$lower.10 > 1, 1, p1.s$upper.10)

p1.s$inside <- ifelse(round(p1.s$p.inference, 2) >= round(p1.s$lower.b, 2) & round(p1.s$p.inference, 2) <= round(p1.s$upper.b, 2), 1, 0)
p1.s$inside.5 <- ifelse(round(p1.s$p.inference, 2) >= round(p1.s$lower.5, 2) & round(p1.s$p.inference, 2) <= round(p1.s$upper.5, 2), 1, 0)
p1.s$inside.10 <- ifelse(round(p1.s$p.inference, 2) >= round(p1.s$lower.10, 2) & round(p1.s$p.inference, 2) <= round(p1.s$upper.10, 2), 1, 0)

p1.s <- within(p1.s, {
  p.inference <- round(p.inference, 2)
  lower.b <- round(lower.b, 2)
  upper.b <- round(upper.b, 2)
  lower.5 <- round(lower.5, 2)
  lower.5 <- ifelse(lower.5 < 0, 0, lower.5)
  upper.5 <- round(upper.5, 2)
  upper.5 <- ifelse(upper.5 > 1, 1, upper.5)
  lower.10 <- round(lower.10, 2)
  lower.10 <- ifelse(lower.10 < 0, 0, lower.10)
  upper.10 <- round(upper.10, 2)
  upper.10 <- ifelse(upper.10 > 1, 1, upper.10)
})


#### make table:

form <- function(x) substr(formatC(x, digits = 2, format = "f"), 2, 4)
form2 <- function(x) paste0(substr(formatC(round(x*100), digits = 0, format = "d"), 1, 4), "%")

viol.0 <- vapply(split(p1.s, list(p1.s$inference)), function(x, which.l, which.b) str_c("(", form2(mean(x$p.inference < x[,which.l])), ", ", if (!(form(median(abs(x$p.inference[x$p.inference < x[,which.l]] - x[x$p.inference < x[,which.l],which.l])))) == "NA") form(median(abs(x$p.inference[x$p.inference < x[,which.l]] - x[x$p.inference < x[,which.l],which.l]))) else ".00", "; ", form2(mean(x$p.inference > x[,which.b])), ", ", if (!(form(median(abs(x$p.inference[x$p.inference > x[,which.b]] - x[x$p.inference > x[,which.b],which.b])))) == "NA") form(median(abs(x$p.inference[x$p.inference > x[,which.b]] - x[x$p.inference > x[,which.b],which.b]))) else ".00", ")"),  which.l = "lower.b", which.b = "upper.b", "")

viol.0.p <- vapply(split(p1.s, list(p1.s$inference)), function(x, which.l, which.b) str_c(form2(mean(x[,which.b] - x[,which.l]))), which.l = "lower.b", which.b = "upper.b", "")

viol.5 <- vapply(split(p1.s, list(p1.s$inference)), function(x, which.l, which.b) str_c("(", form2(mean(x$p.inference < x[,which.l])), ", ", if (!(form(median(abs(x$p.inference[x$p.inference < x[,which.l]] - x[x$p.inference < x[,which.l],which.l])))) == "NA") form(median(abs(x$p.inference[x$p.inference < x[,which.l]] - x[x$p.inference < x[,which.l],which.l]))) else ".00", "; ", form2(mean(x$p.inference > x[,which.b])), ", ", if (!(form(median(abs(x$p.inference[x$p.inference > x[,which.b]] - x[x$p.inference > x[,which.b],which.b])))) == "NA") form(median(abs(x$p.inference[x$p.inference > x[,which.b]] - x[x$p.inference > x[,which.b],which.b]))) else ".00", ")"),  which.l = "lower.5", which.b = "upper.5", "")

viol.5.p <- vapply(split(p1.s, list(p1.s$inference)), function(x, which.l, which.b) str_c(form2(mean(x[,which.b] - x[,which.l]))), which.l = "lower.5", which.b = "upper.5", "")

viol.10 <- vapply(split(p1.s, list(p1.s$inference)), function(x, which.l, which.b) str_c("(", form2(mean(x$p.inference < x[,which.l])), ", ", if (!(form(median(abs(x$p.inference[x$p.inference < x[,which.l]] - x[x$p.inference < x[,which.l],which.l])))) == "NA") form(median(abs(x$p.inference[x$p.inference < x[,which.l]] - x[x$p.inference < x[,which.l],which.l]))) else ".00", "; ", form2(mean(x$p.inference > x[,which.b])), ", ", if (!(form(median(abs(x$p.inference[x$p.inference > x[,which.b]] - x[x$p.inference > x[,which.b],which.b])))) == "NA") form(median(abs(x$p.inference[x$p.inference > x[,which.b]] - x[x$p.inference > x[,which.b],which.b]))) else ".00", ")"),  which.l = "lower.10", which.b = "upper.10", "")

viol.10.p <- vapply(split(p1.s, list(p1.s$inference)), function(x, which.l, which.b) str_c(form2(mean(x[,which.b] - x[,which.l]))), which.l = "lower.10", which.b = "upper.10", "")

t.out <- as.data.frame(t(paste(form2(aggregate(inside~inference, p1.s, mean)[,2]), viol.0)))
t.out <- rbind(t.out, as.data.frame(t(paste(form2(aggregate(inside.5~inference, p1.s, mean)[,2]), viol.5))))
t.out <- rbind(t.out, as.data.frame(t(paste(form2(aggregate(inside.10~inference, p1.s, mean)[,2]), viol.10))))
colnames(t.out) <- inferences
t.out <- cbind(border = c("+/- 0", "+/- 5", "+/- 10"), t.out)

suppressWarnings(print(ascii(t.out, include.rownames=FALSE, caption = "Percentages of responses being p-consistent (lower violations/median, upper violations/median; median is the median absolute distance from the border)")))

aggregate(I(upper.b - lower.b) ~ inference, p1.s, mean)
aggregate(I(upper.5 - lower.5) ~ inference, p1.s, mean)
aggregate(I(upper.10 - lower.10) ~ inference, p1.s, mean)

#### mixed model analysis:
  
p1.s <- within(p1.s, {
  diff.0 <- inside - (upper.b - lower.b)
  diff.5 <- inside.5 - (upper.5 - lower.5)
  diff.10 <- inside.10 - (upper.10 - lower.10)
})


(coh.lmm1 <- mixed(diff.0 ~ inference + (1|code) + (inference|item), p1.s, control = lmer.controls, test.intercept=TRUE))

K2 <- head(model.matrix(coh.lmm1[[2]]), 4)
rownames(K2) <- as.character(p1.s$inference[1:4])
K2

summary(glht(coh.lmm1[[2]], linfct = K2, alternative = "greater"), test = adjusted("holm"))
summary(glht(coh.lmm1[[2]], linfct = K2, alternative = "greater"), test = adjusted("none"))
  
coh.lmm2 <- mixed(diff.5 ~ inference + (1|code) + (inference|item), p1.s, control = lmer.controls, test.intercept=TRUE)
summary(glht(coh.lmm2[[2]], linfct = K2, alternative = "greater"), test = adjusted("holm"))
summary(glht(coh.lmm2[[2]], linfct = K2, alternative = "greater"), test = adjusted("none"))
  
coh.lmm3 <- mixed(diff.10 ~ inference + (1|code) + (inference|item), p1.s, control = lmer.controls, test.intercept=TRUE)
summary(glht(coh.lmm3[[2]], linfct = K2, alternative = "greater"), test = adjusted("holm"))
summary(glht(coh.lmm3[[2]], linfct = K2, alternative = "greater"), test = adjusted("none"))



#############
# Deductive Task
#############

p2.files <- list.files("data", "rmc1_abstract_", full.names=TRUE)

p2.in <- do.call("rbind", lapply(p2.files, read.csv2, dec = ".", fileEncoding = "UTF-8"))

p2.l <- within(p2.in, {
  trial <- factor(trial)
  item <- factor(item)
  inference <- factor(inference, levels = inferences)
  X <- NULL
})

levels(p2.l2$item) <- c("letter - number", "number - letter")

layout(matrix(1:2, ncol = 1), height = c(3.8, 1))
plotrix::raw.means.plot(p2.l2, "item", "inference", "resp", ylab = "Responses to deductive task", pt.cex= 1.2, legend = FALSE, mar = c(2.7, 4, 0.5, 0.5) + 0.1, lwd = 1.8)
plotrix::raw.means.plot(p2.l2, "item", "inference", "resp", ylab = "Responses to deductive task",pt.cex= 1, title = "Conditional", plot = FALSE, yjust = 0.48, xjust = 0.3, l.bty = "o", lwd = 1.8)

p2.l2 <- within(p2.l, resp <- resp/100)
abst.m1 <- mixed(resp ~ inference * conditional + (inference+conditional|code), p2.l2, control = lmerControl(check.nobs.vs.nRE = "ignore", optCtrl = list(maxfun = 100000), optimizer="Nelder_Mead"), test.intercept=TRUE)

tmp <- anova(abst.m1)[1:6]
tmp$p.value <- round_ps(tmp$p.value)
tmp$F <- formatC(tmp$F, format = "f", digits = 2)
tmp$ddf <- formatC(tmp$ddf, format = "d")
tmp$F.scaling <- formatC(tmp$F.scaling, format = "f", digits = 2)
tmp


contr <- rbind("MP vs MT" = c(1, -1, 0, 0),
               "MT vs AC&DA" = c(0, 2, -1, -1),
               "AC vs DA" = c(0, 0, 1, -1))
x <- summary(glht(abst.m1[[2]], linfct = mcp(inference = contr), alternative = "greater"), test = adjusted("none"))
#str(x)
round(as.data.frame(unclass(x$test)[-c(1,2, 7)]), 4)
